
/*
 * DEFS.H
 *
 *
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution
 *    for specific rights granted.
 */

#include "config.h"

#include <sys/types.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/socket.h>		/* internet sockets	*/
#include <sys/un.h>		/* unix domain sockets	*/
#include <sys/mman.h>		/* mmap()		*/
#ifdef _AIX
#include <sys/select.h>
#endif
#if USE_SYSV_DIR
#include <dirent.h>
typedef struct dirent den_t;
#else
#include <sys/dir.h>
typedef struct direct den_t;
#endif
#include <netinet/in.h>		/* internet sockets	*/
#include <netinet/tcp.h>	/* TCP_NODELAY sockopt  */
#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#if USE_BSTRING_H
#include <bstring.h>
#endif

#if HAS_PROC_TITLE
#include <libutil.h>
#endif

#if USE_POLL
#include <poll.h>		/* poll()		*/
#endif

#if USE_PCOMMIT_SHM || USE_SPAM_SHM || USE_CANCEL_SHM
#include <sys/ipc.h>		/* SYSV shared memory	*/
#include <sys/shm.h>		/* SYSV shared memory	*/
#endif

#include <stdarg.h>
#include <stddef.h>
#if USE_STRINGS_H
#include <strings.h>
#endif
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <netdb.h>		/* internet sockets	*/
#include <arpa/inet.h>
#include <signal.h>
#include <syslog.h>
#include <pwd.h>
#include <grp.h>
#include <math.h>
#include <fcntl.h>

#ifdef	POST_HIDENNTPPOSTHOST
#include	<md5.h>
#endif
#ifdef	POST_CRYPTXTRACE
#include	<des.h>
#endif
#ifdef	RADIUS_ENABLED
#include	<radlib.h>
#endif

typedef signed char	int8;
typedef short		int16;
typedef int		int32;
typedef unsigned char	uint8;
typedef unsigned short	uint16;
typedef unsigned int	uint32;

#define Prototype extern
#define arysize(ary)	(sizeof(ary)/sizeof((ary)[0]))

/*
 * diablo.config
 */

#define EXPIRE_FEEDER	0
#define EXPIRE_READER	1

#define HASH_PRIME	0
#define HASH_CRC	1
#define HASH_OCRC	2

#define ACTIVE_OFF	0
#define ACTIVE_ON	1

/*
 * History
 */

typedef struct {
    int32	h1;
    int32	h2;
} hash_t;

typedef struct HistHead {
    uint32	hmagic;		/* 0xA1B2C3D4			*/
    uint32	hashSize;	/* entries in hash table	*/
    uint16	version;	/* version of history file	*/
    uint16	henSize;	/* size of history entry	*/
    uint16	headSize;	/* size of history header	*/
} HistHead;

#define HMAGIC		((uint32)0xA1B2C3D4)
#define HVERSION	1

/*
 * History file entry.  The filename is calculated from the gmt start
 * (calculates the directory), and file id.  Each incoming process
 * allocates its own file & id and appends to it.
 */

typedef struct History {
    uint32 	next;	/* next link		*/
    uint32	gmt;	/* expire slot number	*/
    hash_t	hv;	/* hash value		*/
    uint16 	iter;	/* file id 		*/
    uint16	exp;	/* hours relative to gmt minutes */
    uint32	boffset;/* starting offset in file	 */
    int32	bsize;	/* size of file		*/
} History;

#define EXPF_HEADONLY	0x8000	/* header-only flag (in exp field) */
#define EXPF_MASK	0x7FFF
#define EXPF_FLAGS	0x8000
#define H_EXP(n)	((uint16)((n) & EXPF_MASK))

/*
 * Pre/PostCommit cache
 */

#define PC_PRECOMM		0x001
#define PC_POSTCOMM		0x002
#define PC_DELCOMM		0x004

/*
 * FeedValid()
 */

#define FEED_VALID		0
#define FEED_MAXCONNECT		-1
#define FEED_MISSINGLABEL	-2

#define OD_HARTS        256     /* modulo for data files, POWER OF 2    */
#define OD_HMASK        (OD_HARTS-1)

/*
 * RunProgPipe()
 */

#define RPF_STDIN	0x0001
#define RPF_STDOUT	0x0002
#define RPF_STDERR	0x0004


#define HGF_FAST	0x01
#define HGF_NOSEARCH	0x02

typedef struct Node {
    struct Node		*no_Next;
    int32		no_Value;
    const char		*no_Name;
    const char		*no_Name2;
    void		*no_Data;
} Node;

#define MAXLINE		16384
#define MAXMSGIDLEN	512
#define MAXGNAME	256
#define MAXFORKS	256
#define MAXFEEDS	128
#define MAXFDS		(MAXFORKS+MAXFEEDS+32)
#define MAXDIABLOFDCACHE 8
#if MAXFDS > FD_SETSIZE
#undef MAXFDS
#define MAXFDS FD_SETSIZE
#endif


#ifndef HSIZE

#if USE_SMALL_HASH
#define HSIZE		(1024*1024)		
#else
#if USE_LARGE_HASH
#define HSIZE		(4096*1024)
#else
#if USE_REALLY_LARGE_HASH
#define HSIZE		(8192*1024)
#else
#define HSIZE		(4096*1024)
#endif
#endif
#endif

#endif

/*
 * Default history retention, in days.   Articles are rejected if they
 * are older the the specified number of days.  History retention is
 * 2 days longer.
 */

#ifndef REMEMBERDAYS
#if USE_SHORT_REMEMBER
#define REMEMBERDAYS	7
#else
#define REMEMBERDAYS	14
#endif
#endif

#define MAXSIMUL	5	/* maximum simultanious from same label */

#define RCOK		0	/* article accepted		*/
#define RCALREADY	1	/* we already have this article	*/
#define RCTRYAGAIN	2	/* something went wrong, requeue it for later */
#define RCREJECT	3	/* there is something wrong with your article */
#define RCERROR		4	/* an unexpected error occured  */

#define XLOCK_SH	1
#define XLOCK_EX	2
#define XLOCK_UN	3
#define XLOCK_NB	0x10

#define MAXFEEDTABLE	256

typedef struct Buffer {
    int			bu_Beg;
    int			bu_NLScan;
    int			bu_End;
    int			bu_BufMax;	/* normal maximum	*/
    int			bu_DataMax;	/* extended maximum	*/
    int			bu_Fd;
    int			bu_Error;
    char		*bu_Data;	/* included/extended buffer	*/
    char		bu_Buf[1024];	/* included buffer		*/
} Buffer;

#define XADV_WILLNEED	1
#define XADV_SEQUENTIAL	2

#include <lib/mem.h>
#include <lib/kpdb.h>
#include <lib/expire.h>
#include <lib/ctl.h>

/*
 * MISC
 */

#ifndef LOG_PERROR
#define LOG_PERROR	0
#endif
#ifndef INADDR_NONE
#define INADDR_NONE	((unsigned long)-1)
#endif

#ifndef TEST
#include <obj/lib-protos.h>
#endif

/*
 * can't use Prototype stuff for functions which may
 * already be prototyped.
 */

#if USE_STRERROR
char *strerror(int e);
#endif

/*
 * hack
 */

#if USE_MEMMOVE
void *memmove(char *dst0, const char *src0, size_t length);
#endif

/*
 * If setproctitle() exists, stprintf() is macro'd to it directly, otherwise
 * we use argv stuffing.
 */

#if HAS_PROC_TITLE
#define stprintf	setproctitle
#endif

/*
 * If we don't have snprintf() we have to supply it.  snprintf() is 
 * absolutely critical.
 */

#if HAVE_SNPRINTF == 0

int snprintf(char *str, size_t size, const char *format, ...);
int vsnprintf(char *str, size_t size, const char *format, va_list ap);

#endif

#if HAVE_STRSEP == 0

char *strsep(register char **stringp, register const char *delim);

#endif

#if USE_INTERNAL_VSYSLOG

void vsyslog(int priority, const char *msg, va_list va);

#endif

